package l06_interfaces;

import java.util.Map;
import java.util.HashMap;

public class ShoppingCartExample {
	public static void main(String... args) {

		IShoppingCart shoppingCart = new NormalShoppingCart();

		shoppingCart.addItem(new Item("tx7687bb", "mobile phone x", 235.99));
		shoppingCart.addItem(new Item("rt3434uy", "mobile phone y", 279.99));
		shoppingCart.addItem(new Item("su4563dd", "mobile phone z", 461.99));
		System.out.print(shoppingCart);
		System.out.println("total order amount " + shoppingCart.calculateTotal());
		shoppingCart.removeItem("tx7687bb");
		System.out.println();
		System.out.print(shoppingCart);
		System.out.println("total order amount " + shoppingCart.calculateTotal());
	}

	record Item(String id, String name, double price) {
		@Override
		public String toString() {
			return "{ " + id + ", " + name + ", " + price + " }";
		}
	}

	interface IShoppingCart {
		void addItem(Item item);
		void removeItem(String itemId);
		double calculateTotal();
	}

	static class NormalShoppingCart implements IShoppingCart {

		Map<String, Item> cartItems = new HashMap<>();

		@Override
		public void addItem(Item item) {
			cartItems.put(item.id(), item);
		}

		@Override
		public void removeItem(String itemId) {
			cartItems.remove(itemId);
		}

		@Override
		public double calculateTotal() {
			double totalOrderAmt = 0;
			for(String id : cartItems.keySet()) {
				totalOrderAmt += cartItems.get(id).price();
			}
			return totalOrderAmt;
		}

		@Override
		public String toString() {
			StringBuilder stringBuffer = new StringBuilder();
			cartItems.forEach((key, value) ->
				stringBuffer.append(key).append(": ").append(value).append("\n"));
			return stringBuffer.toString();
		}
	}
}

